@@ -14,8 +14,10 @@ from operator import itemgetter
 from copy import deepcopy
 
 from label_studio_converter.utils import (
-    parse_config, create_tokens_and_tags, download, get_image_size, get_image_size_and_channels, ensure_dir
+    parse_config, create_tokens_and_tags, download, get_image_size, get_image_size_and_channels, ensure_dir,
+    get_polygon_area, get_polygon_bounding_box
 )
+from label_studio_converter import brush
 
 
 logger = logging.getLogger(__name__)
@@ -33,6 +35,8 @@ class Format(Enum):
     CONLL2003 = 5
     COCO = 6
     VOC = 7
+    BRUSH_TO_NUMPY = 8
+    BRUSH_TO_PNG = 9
 
     def __str__(self):
         return self.name
@@ -84,6 +88,12 @@ class Converter(object):
         elif format == Format.VOC:
             image_dir = kwargs.get('image_dir')
             self.convert_to_voc(input_data, output_data, output_image_dir=image_dir, is_dir=is_dir)
+        elif format == Format.BRUSH_TO_NUMPY:
+            image_dir = kwargs.get('image_dir')
+            brush.convert_task_dir(input_data, output_data, out_format='numpy')
+        elif format == Format.BRUSH_TO_PNG:
+            image_dir = kwargs.get('image_dir')
+            brush.convert_task_dir(input_data, output_data, out_format='png')
 
     def _get_data_keys_and_output_tags(self, output_tags=None):
         data_keys = set()
@@ -117,8 +127,14 @@ class Converter(object):
         if not ('Text' in input_tag_types and 'Labels' in output_tag_types):
             all_formats.remove(Format.CONLL2003.name)
         if not ('Image' in input_tag_types and 'RectangleLabels' in output_tag_types):
-            all_formats.remove(Format.COCO.name)
             all_formats.remove(Format.VOC.name)
+        if not ('Image' in input_tag_types and ('RectangleLabels' in output_tag_types or
+                                                'PolygonLabels' in output_tag_types)):
+            all_formats.remove(Format.COCO.name)
+        if not ('Image' in input_tag_types and 'BrushLabels' in output_tag_types):
+            all_formats.remove(Format.BRUSH_TO_NUMPY.name)
+            all_formats.remove(Format.BRUSH_TO_PNG.name)
+
         return all_formats
 
     @property
@@ -148,14 +164,13 @@ class Converter(object):
                            'where value is list of dicts')
         result = []
         if 'completions' in d:
-            if len(d['completions']) != 1:
-                raise NotImplementedError(
-                    'Currently only one completion could be added per task - '
-                    'we can\'t convert more than one completions, but {num_completions} found in item: {item}'.format(
-                        num_completions=len(d['completions']), item=json.dumps(d, indent=2)))
-            if d['completions'][0].get('skipped'):
+            # get last not skipped completion and make result from it
+            tmp = list(filter(lambda x: not (x.get('skipped', False) or x.get('was_cancelled', False)), d['completions']))
+            if len(tmp) > 0:
+                result = sorted(tmp, key=lambda x: x['created_at'], reverse=True)[0]['result']
+            else:
                 return None
-            result = d['completions'][0]['result']
+            
         elif 'result' in d:
             result = d['result']
 
@@ -171,6 +186,7 @@ class Converter(object):
                     v['original_height'] = r['original_height']
                 outputs[r['from_name']].append(v)
         return {
+            'id': d['id'],
             'input': inputs,
             'output': outputs
         }
@@ -210,6 +226,8 @@ class Converter(object):
         item_iterator = self.iter_from_dir if is_dir else self.iter_from_json_file
         for item in item_iterator(input_data):
             record = deepcopy(item['input'])
+            if item.get('id') is not None:
+                record['id'] = item['id']
             for name, value in item['output'].items():
                 record[name] = self._prettify(value)
             records.append(record)
@@ -225,6 +243,8 @@ class Converter(object):
         item_iterator = self.iter_from_dir if is_dir else self.iter_from_json_file
         for item in item_iterator(input_data):
             record = deepcopy(item['input'])
+            if item.get('id') is not None:
+                record['id'] = item['id']
             for name, value in item['output'].items():
                 pretty_value = self._prettify(value)
                 record[name] = pretty_value if isinstance(pretty_value, str) else json.dumps(pretty_value)
@@ -279,11 +299,11 @@ class Converter(object):
                     logger.error('Unable to download {image_path}. The item {item} will be skipped'.format(
                         image_path=image_path, item=item
                     ), exc_info=True)
-            bboxes = next(iter(item['output'].values()))
-            if len(bboxes) == 0:
+            labels = next(iter(item['output'].values()))
+            if len(labels) == 0:
                 logger.error('Empty bboxes.')
                 continue
-            width, height = bboxes[0]['original_width'], bboxes[0]['original_height']
+            width, height = labels[0]['original_width'], labels[0]['original_height']
             image_id = len(images)
             images.append({
                 'width': width,
@@ -292,8 +312,14 @@ class Converter(object):
                 'file_name': image_path
             })
 
-            for bbox in bboxes:
-                category_name = bbox['rectanglelabels'][0]
+            for label in labels:
+                if 'rectanglelabels' in label:
+                    category_name = label['rectanglelabels'][0]
+                elif 'polygonlabels' in label:
+                    category_name = label['polygonlabels'][0]
+                else:
+                    raise ValueError("Unknown label type")
+
                 if category_name not in category_name_to_id:
                     category_id = len(categories)
                     category_name_to_id[category_name] = category_id
@@ -302,21 +328,41 @@ class Converter(object):
                         'name': category_name
                     })
                 category_id = category_name_to_id[category_name]
+
                 annotation_id = len(annotations)
-                x = int(bbox['x'] / 100 * width)
-                y = int(bbox['y'] / 100 * height)
-                w = int(bbox['width'] / 100 * width)
-                h = int(bbox['height'] / 100 * height)
-                annotations.append({
-                    'id': annotation_id,
-                    'image_id': image_id,
-                    'category_id': category_id,
-                    'segmentation': [],
-                    'bbox': [x, y, w, h],
-                    'ignore': 0,
-                    'iscrowd': 0,
-                    'area': w * h
-                })
+
+                if "rectanglelabels" in label:
+                    x = int(label['x'] / 100 * width)
+                    y = int(label['y'] / 100 * height)
+                    w = int(label['width'] / 100 * width)
+                    h = int(label['height'] / 100 * height)
+
+                    annotations.append({
+                        'id': annotation_id,
+                        'image_id': image_id,
+                        'category_id': category_id,
+                        'segmentation': [],
+                        'bbox': [x, y, w, h],
+                        'ignore': 0,
+                        'iscrowd': 0,
+                        'area': w * h
+                    })
+                elif "polygonlabels" in label:
+                    points_abs = [(x / 100 * width, y / 100 * height) for x, y in label["points"]]
+                    x, y = zip(*points_abs)
+
+                    annotations.append({
+                        'id': annotation_id,
+                        'image_id': image_id,
+                        'category_id': category_id,
+                        'segmentation': [[coord for point in points_abs for coord in point]],
+                        'bbox': get_polygon_bounding_box(x, y),
+                        'ignore': 0,
+                        'iscrowd': 0,
+                        'area': get_polygon_area(x, y)
+                    })
+                else:
+                    raise ValueError("Unknown label type")
 
         with io.open(output_file, mode='w', encoding='utf8') as fout:
             json.dump({
@@ -365,8 +411,19 @@ class Converter(object):
                 except:
                     logger.error('Unable to download {image_path}. The item {item} will be skipped'.format(
                         image_path=image_path, item=item), exc_info=True)
-                    continue
-            width, height, channels = get_image_size_and_channels(image_path)
+                    # On error, use default number of channels
+                    channels = 3
+                else:
+                    # retrieve number of channels from downloaded image
+                    _, _, channels = get_image_size_and_channels(image_path)
+
+            bboxes = next(iter(item['output'].values()))
+            if len(bboxes) == 0:
+                logger.error('Empty bboxes.')
+                continue
+
+            width, height = bboxes[0]['original_width'], bboxes[0]['original_height']
+
             image_name = os.path.splitext(os.path.basename(image_path))[0]
             xml_filepath = os.path.join(annotations_dir, image_name + '.xml')
 
@@ -393,7 +450,7 @@ class Converter(object):
             root_node.appendChild(size_node)
             create_child_node(doc, 'segmented', '0', root_node)
 
-            bboxes = next(iter(item['output'].values()))
+
             for bbox in bboxes:
                 name = bbox['rectanglelabels'][0]
                 x = int(bbox['x'] / 100 * width)
