@@ -4,6 +4,8 @@ import xml.etree.ElementTree
 import requests
 import hashlib
 import logging
+import urllib
+import numpy as np
 
 from operator import itemgetter
 from PIL import Image
@@ -15,7 +17,7 @@ logger = logging.getLogger(__name__)
 def tokenize(text):
     tok_start = 0
     out = []
-    for tok in text.split(' '):
+    for tok in text.split():
         if len(tok):
             out.append((tok, tok_start))
             tok_start += len(tok) + 1
@@ -57,6 +59,12 @@ def create_tokens_and_tags(text, spans):
 
 
 def download(url, output_dir, filename=None):
+    if '/data/' in url and '?d=' in url:
+        filename, dir_path = url.split('/data/', 1)[-1].split('?d=')
+        dir_path = str(urllib.parse.unquote(dir_path))
+        filepath = os.path.join(dir_path, filename)
+        return filepath, False
+
     if filename is None:
         filename = hashlib.md5(url.encode()).hexdigest()
     filepath = os.path.join(output_dir, filename)
@@ -66,7 +74,7 @@ def download(url, output_dir, filename=None):
         r.raise_for_status()
         with io.open(filepath, mode='wb') as fout:
             fout.write(r.content)
-    return filepath
+    return filepath, True
 
 
 def get_image_size(image_path):
@@ -112,3 +120,18 @@ def parse_config(config_string):
             tag_info['inputs'].append(inputs[input_tag_name])
 
     return outputs
+
+
+def get_polygon_area(x, y):
+    """https://en.wikipedia.org/wiki/Shoelace_formula"""
+
+    assert len(x) == len(y)
+
+    return float(0.5 * np.abs(np.dot(x, np.roll(y, 1)) - np.dot(y, np.roll(x, 1))))
+
+
+def get_polygon_bounding_box(x, y):
+    assert len(x) == len(y)
+
+    x1, y1, x2, y2 = min(x), min(y), max(x), max(y)
+    return [x1, y1, x2 - x1, y2 - y1]
